@@ -1301,10 +1301,10 @@ void FGAIAircraft::updateActualState(double dt)
     roll = _performance->actualBankAngle(this, tgt_roll, dt);
 
     // adjust altitude (meters) based on current vertical speed (fpm)
-    altitude_ft += vs / 60.0 * dt;
+    altitude_ft += vs_fps * dt;
     pos.setElevationFt(altitude_ft);
 
-    vs = _performance->actualVerticalSpeed(this, tgt_vs, dt);
+    vs_fps = _performance->actualVerticalSpeed(this, tgt_vs, dt) / 60;
     pitch = _performance->actualPitch(this, tgt_pitch, dt);
 }
 
@@ -1328,6 +1328,14 @@ bool FGAIAircraft::reachedEndOfCruise(double &distance) {
     }
     
     if (curr->getName() == string("BOD")) {
+        // Sentry: FLIGHTGEAR-893
+        if (!trafficRef->getArrivalAirport()) {
+            SG_LOG(SG_AI, SG_WARN, trafficRef->getCallSign() << "FGAIAircraft::reachedEndOfCruise: no arrival airport");
+            setDie(true);
+            // return 'done' here, we are broken
+            return true;
+        }
+        
         double dist = fp->getDistanceToGo(pos.getLatitudeDeg(), pos.getLongitudeDeg(), curr);
         double descentSpeed = (getPerformance()->vDescent() * SG_NM_TO_METER) / 3600.0;     // convert from kts to meter/s
         double descentRate  = (getPerformance()->descentRate() * SG_FEET_TO_METER) / 60.0;  // convert from feet/min to meter/s
