@@ -32,17 +32,19 @@
 #include <osg/Node>
 #include <osgDB/FileUtils>
 
+#include <simgear/debug/ErrorReportingCallback.hxx>
+#include <simgear/debug/logstream.hxx>
 #include <simgear/misc/sg_path.hxx>
+#include <simgear/props/props.hxx>
 #include <simgear/scene/model/modellib.hxx>
 #include <simgear/scene/util/SGNodeMasks.hxx>
-#include <simgear/debug/logstream.hxx>
-#include <simgear/props/props.hxx>
 
-#include <Main/globals.hxx>
+#include <Main/ErrorReporter.hxx>
 #include <Main/fg_props.hxx>
+#include <Main/globals.hxx>
 #include <Scenery/scenery.hxx>
-#include <Scripting/NasalSys.hxx>
 #include <Scripting/NasalModelData.hxx>
+#include <Scripting/NasalSys.hxx>
 #include <Sound/fg_fx.hxx>
 
 #include "AIFlightPlan.hxx"
@@ -68,6 +70,24 @@ public:
 
     FGAIModelData* clone() const override { return new FGAIModelData(); }
 
+    ErrorContext getErrorContext() const override
+    {
+        return _errorContext;
+    }
+
+    void addErrorContext(const std::string& key, const std::string& value)
+    {
+        _errorContext[key] = value;
+    }
+
+    void captureErrorContext(const std::string& key)
+    {
+        const auto v = flightgear::ErrorReporter::threadSpecificContextValue(key);
+        if (!v.empty()) {
+            addErrorContext(key, v);
+        }
+    }
+
     /** osg callback, thread-safe */
     void modelLoaded(const std::string& path, SGPropertyNode *prop, osg::Node *n)
     {
@@ -120,6 +140,8 @@ private:
     bool _interiorLoaded = false;
     float _radius = -1.0;
     SGPropertyNode* _root;
+
+    ErrorContext _errorContext;
 };
 
 FGAIBase::FGAIBase(object_type ot, bool enableHot) :
@@ -136,7 +158,7 @@ FGAIBase::FGAIBase(object_type ot, bool enableHot) :
     _otype(ot)
 {
     tgt_heading = hdg = tgt_altitude_ft = tgt_speed = 0.0;
-    tgt_roll = roll = tgt_pitch = tgt_yaw = tgt_vs = vs = pitch = 0.0;
+    tgt_roll = roll = tgt_pitch = tgt_yaw = tgt_vs = vs_fps = pitch = 0.0;
     bearing = elevation = range = rdot = 0.0;
     x_shift = y_shift = rotation = 0.0;
     in_range = false;
@@ -227,6 +249,11 @@ FGAIBase::removeModel()
     }
 }
 
+void FGAIBase::setScenarioPath(const std::string& scenarioPath)
+{
+    _scenarioPath = scenarioPath;
+}
+
 void FGAIBase::readFromScenario(SGPropertyNode* scFileNode)
 {
     if (!scFileNode)
@@ -307,6 +334,11 @@ void FGAIBase::update(double dt) {
             const string& fxpath = _modeldata->get_sound_path();
             if (fxpath != "")
             {
+                simgear::ErrorReportContext ec("ai-model", _name);
+                if (!_scenarioPath.empty()) {
+                    ec.add("scenario-path", _scenarioPath);
+                }
+
                 props->setStringValue("sim/sound/path", fxpath.c_str());
 
                 // Remove any existing sound FX (e.g. from another model)
@@ -603,6 +635,15 @@ bool FGAIBase::init(ModelSearchOrder searchOrder)
 
     props->addChild("type")->setStringValue("AI");
     _modeldata = new FGAIModelData(props);
+    _modeldata->addErrorContext("ai", _name);
+    _modeldata->captureErrorContext("scenario-path");
+
+    // set by FGAISchedule::createAIAircraft
+    _modeldata->captureErrorContext("traffic-aircraft-callsign");
+
+    if (_otype == otMultiplayer) {
+        _modeldata->addErrorContext("multiplayer", getCallSign());
+    }
 
     vector<string> model_list = resolveModelPath(searchOrder);
     _model= SGModelLib::loadPagedModel(model_list, props, _modeldata);
@@ -949,7 +990,7 @@ double FGAIBase::_getRdot() const {
 }
 
 double FGAIBase::_getVS_fps() const {
-    return vs/60.0;
+    return vs_fps;
 }
 
 double FGAIBase::_get_speed_east_fps() const {
@@ -961,7 +1002,7 @@ double FGAIBase::_get_speed_north_fps() const {
 }
 
 void FGAIBase::_setVS_fps( double _vs ) {
-    vs = _vs*60.0;
+    vs_fps = _vs;
 }
 
 double FGAIBase::_getAltitude() const {
