@@ -17,9 +17,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
+#include <config.h>
 
 #include <algorithm>
 #include <string>
@@ -35,11 +33,13 @@
 
 #include "AICarrier.hxx"
 
-FGAICarrier::FGAICarrier() : FGAIShip(otCarrier), deck_altitude(65.0065) {
+FGAICarrier::FGAICarrier() :
+    FGAIShip(otCarrier),
+    deck_altitude(65.0065)
+{
 }
 
-FGAICarrier::~FGAICarrier() {
-}
+FGAICarrier::~FGAICarrier() = default;
 
 void FGAICarrier::readFromScenario(SGPropertyNode* scFileNode) {
   if (!scFileNode)
@@ -67,11 +67,14 @@ void FGAICarrier::readFromScenario(SGPropertyNode* scFileNode) {
     // Transform to the right coordinate frame, configuration is done in
     // the usual x-back, y-right, z-up coordinates, computations
     // in the simulation usual body x-forward, y-right, z-down coordinates
-    flols_off(0) = - flols->getDoubleValue("x-offset-m", 0);
-    flols_off(1) = flols->getDoubleValue("y-offset-m", 0);
-    flols_off(2) = - flols->getDoubleValue("z-offset-m", 0);
+    _flolsPosOffset(0) = - flols->getDoubleValue("x-offset-m", 0);
+    _flolsPosOffset(1) = flols->getDoubleValue("y-offset-m", 0);
+    _flolsPosOffset(2) = - flols->getDoubleValue("z-offset-m", 0);
+
+    _flolsHeadingOffsetDeg = flols->getDoubleValue("heading-offset-deg", 0.0);
+    _flolsApproachAngle = flols->getDoubleValue("glidepath-angle-deg", 3.5);
   } else
-    flols_off = SGVec3d::zeros();
+    _flolsPosOffset = SGVec3d::zeros();
 
   std::vector<SGPropertyNode_ptr> props = scFileNode->getChildren("parking-pos");
   std::vector<SGPropertyNode_ptr>::const_iterator it;
@@ -180,13 +183,13 @@ void FGAICarrier::update(double dt) {
     // rotate the eyepoint wrt carrier vector into the carriers frame
     eyeWrtCarrier = ec2body.transform(eyeWrtCarrier);
     // the eyepoints vector wrt the flols position
-    SGVec3d eyeWrtFlols = eyeWrtCarrier - flols_off;
+    SGVec3d eyeWrtFlols = eyeWrtCarrier - _flolsPosOffset;
 
     // the distance from the eyepoint to the flols
     dist = norm(eyeWrtFlols);
 
     // now the angle, positive angles are upwards
-    if (fabs(dist) < SGLimits<float>::min()) {
+    if (fabs(dist) < SGLimits<double>::min()) {
       angle = 0;
     } else {
       double sAngle = -eyeWrtFlols(2)/dist;
@@ -320,6 +323,12 @@ void FGAICarrier::bind() {
     props->setStringValue("sign", sign.c_str());
     props->setBoolValue("controls/lighting/deck-lights", false);
     props->setDoubleValue("controls/lighting/flood-lights-red-norm", 0);
+
+    // Write out a list of the parking positions - useful for the UI to select
+    // from
+    for (const auto& ppos : ppositions) {
+        if (ppos.name != "") props->addChild("parking-pos")->setStringValue("name", ppos.name);
+    }
 }
 
 bool FGAICarrier::getParkPosition(const string& id, SGGeod& geodPos,
@@ -327,11 +336,9 @@ bool FGAICarrier::getParkPosition(const string& id, SGGeod& geodPos,
 {
 
     // FIXME: does not yet cover rotation speeds.
-    list<ParkPosition>::iterator it = ppositions.begin();
-    while (it != ppositions.end()) {
+    for (const auto& ppos : ppositions) {
         // Take either the specified one or the first one ...
-        if ((*it).name == id || id.empty()) {
-            ParkPosition ppos = *it;
+        if (ppos.name == id || id.empty()) {
             SGVec3d cartPos = getCartPosAt(ppos.offset);
             geodPos = SGGeod::fromCart(cartPos);
             hdng = hdg + ppos.heading_deg;
@@ -341,12 +348,28 @@ bool FGAICarrier::getParkPosition(const string& id, SGGeod& geodPos,
             uvw = SGVec3d(chdng*speed_fps, shdng*speed_fps, 0);
             return true;
         }
-        ++it;
     }
 
     return false;
 }
 
+bool FGAICarrier::getFLOLSPositionHeading(SGGeod& geodPos, double& heading) const
+{
+    SGVec3d cartPos = getCartPosAt(_flolsPosOffset);
+    geodPos = SGGeod::fromCart(cartPos);
+
+    // at present we don't support a heading offset for the FLOLS, so
+    // heading is just the carrier heading
+    heading = hdg + _flolsHeadingOffsetDeg;
+
+    return true;
+}
+
+double FGAICarrier::getFLOLFSGlidepathAngleDeg() const
+{
+    return _flolsApproachAngle;
+}
+
 // find relative wind
 void FGAICarrier::UpdateWind( double dt) {
 
@@ -371,8 +394,7 @@ void FGAICarrier::UpdateWind( double dt) {
     + (rel_wind_speed_from_north_kts * rel_wind_speed_from_north_kts));
 
     //calculate the relative wind direction
-    rel_wind_from_deg = atan2(rel_wind_speed_from_east_kts, rel_wind_speed_from_north_kts)
-                            * SG_RADIANS_TO_DEGREES;
+    rel_wind_from_deg = SGMiscd::rad2deg(atan2(rel_wind_speed_from_east_kts, rel_wind_speed_from_north_kts));
 
     //calculate rel wind
     rel_wind = rel_wind_from_deg - hdg;
@@ -601,31 +623,35 @@ void FGAICarrier::UpdateJBD(double dt, double jbd_transition_time) {
 
 std::pair<bool, SGGeod> FGAICarrier::initialPositionForCarrier(const std::string& namePennant)
 {
-    SGPropertyNode_ptr aiRoot = fgGetNode("/sim/ai", true);
-    for (const auto scenarioFileNode : aiRoot->getChildren("scenario")) {
-        SGPropertyNode_ptr s = FGAIManager::loadScenarioFile(scenarioFileNode->getStringValue());
-        if (!s || !s->hasChild("scenario")) {
+    FGAIManager::registerScenarios();
+    // this is actually a three-layer search (we want the scenario with the
+    // carrier with the correct penanant or name. Sometimes an XPath for
+    // properties would be quite handy :)
+
+    for (auto s : fgGetNode("/sim/ai/scenarios")->getChildren("scenario")) {
+        auto carriers = s->getChildren("carrier");
+        auto it = std::find_if(carriers.begin(), carriers.end(),
+                               [namePennant] (const SGPropertyNode* n)
+        {
+            // don't want to use a recursive lambda here, so inner search is a flat loop
+            for (auto nameChild : n->getChildren("name")) {
+                if (nameChild->getStringValue() == namePennant) return true;
+            }
+            return false;
+        });
+        if (it == carriers.end()) {
             continue;
         }
-        
-        const std::string carrierType("carrier");
-        SGPropertyNode_ptr scenario = s->getChild("scenario");
-        for (int i = 0; i < scenario->nChildren(); i++) {
-            SGPropertyNode* c = scenario->getChild(i);
-            if (c->getStringValue("type") != carrierType) {
-                continue;
-            }
-            
-            const auto pennant = c->getStringValue("pennant-number");
-            const auto name = c->getStringValue("name");
-            if (pennant == namePennant || name == namePennant) {
-                SGSharedPtr<FGAICarrier> carrier = new FGAICarrier;
-                carrier->readFromScenario(c);
-                return std::make_pair(true, carrier->getGeodPos());
-            }
-        } // of objects in scenario iteration
-    } // of scenario files iteration
-    
+
+        // mark the scenario for loading (which will happen in post-init of the AIManager)
+        fgGetNode("/sim/ai/")->addChild("scenario")->setStringValue(s->getStringValue("id"));
+
+        // read out the initial-position
+        SGGeod geod = SGGeod::fromDeg((*it)->getDoubleValue("longitude"),
+                                      (*it)->getDoubleValue("latitude"));
+        return std::make_pair(true,  geod);
+    } // of scenarios iteration
+
     return std::make_pair(false, SGGeod());
 }
 
@@ -635,8 +661,8 @@ SGSharedPtr<FGAICarrier> FGAICarrier::findCarrierByNameOrPennant(const std::stri
     if (!aiManager) {
         return {};
     }
-    
-    for (const auto aiObject : aiManager->get_ai_list()) {
+
+    for (const auto& aiObject : aiManager->get_ai_list()) {
         if (aiObject->isa(FGAIBase::otCarrier)) {
             SGSharedPtr<FGAICarrier> c = static_cast<FGAICarrier*>(aiObject.get());
             if ((c->sign == namePennant) || (c->_getName() == namePennant)) {
@@ -644,7 +670,42 @@ SGSharedPtr<FGAICarrier> FGAICarrier::findCarrierByNameOrPennant(const std::stri
             }
         }
     } // of all objects iteration
-    
+
     return {};
 }
 
+void FGAICarrier::extractCarriersFromScenario(SGPropertyNode_ptr xmlNode, SGPropertyNode_ptr scenario)
+{
+    for (auto c : xmlNode->getChildren("entry")) {
+        if (c->getStringValue("type") != std::string("carrier"))
+            continue;
+
+        const std::string name = c->getStringValue("name");
+        const std::string pennant = c->getStringValue("pennant-number");
+        if (name.empty() && pennant.empty()) {
+            continue;
+        }
+
+        SGPropertyNode_ptr carrierNode = scenario->addChild("carrier");
+
+        // extract the initial position from the scenario
+        carrierNode->setDoubleValue("longitude", c->getDoubleValue("longitude"));
+        carrierNode->setDoubleValue("latitude", c->getDoubleValue("latitude"));
+
+        // A description of the carrier is also available from the entry.  Primarily for use by the launcher
+        carrierNode->setStringValue("description", c->getStringValue("description"));
+
+        // the find code above just looks for anything called a name (so alias
+        // are possible, for example)
+        if (!name.empty()) carrierNode->addChild("name")->setStringValue(name);
+        if (!pennant.empty()) {
+            carrierNode->addChild("name")->setStringValue(pennant);
+            carrierNode->addChild("pennant-number")->setStringValue(pennant);
+        }
+
+        // extact parkings
+        for (auto p : c->getChildren("parking-pos")) {
+            carrierNode->addChild("parking-pos")->setStringValue(p->getStringValue("name"));
+        }
+    }
+}
