@@ -21,12 +21,13 @@
 #include <cstring>
 #include <algorithm>
 
-#include <simgear/sg_inlines.h>
+#include <simgear/debug/ErrorReportingCallback.hxx>
 #include <simgear/math/sg_geodesy.hxx>
 #include <simgear/props/props_io.hxx>
-#include <simgear/structure/exception.hxx>
-#include <simgear/structure/commands.hxx>
+#include <simgear/sg_inlines.h>
 #include <simgear/structure/SGBinding.hxx>
+#include <simgear/structure/commands.hxx>
+#include <simgear/structure/exception.hxx>
 
 #include <Add-ons/AddonManager.hxx>
 #include <Airports/airport.hxx>
@@ -57,6 +58,7 @@ public:
     Scenario(FGAIManager* man, const std::string& nm, SGPropertyNode* scenarios) :
         _internalName(nm)
     {
+        simgear::ErrorReportContext ec("scenario-name", _internalName);
         for (auto scEntry : scenarios->getChildren("entry")) {
             FGAIBasePtr ai = man->addObject(scEntry);
             if (ai) {
@@ -74,9 +76,15 @@ public:
         if (!loadScript.empty()) {
             FGNasalSys* nasalSys = globals->get_subsystem<FGNasalSys>();
             std::string moduleName = "scenario_" + _internalName;
-            nasalSys->createModule(moduleName.c_str(), moduleName.c_str(),
-                                   loadScript.c_str(), loadScript.size(),
-                                   nullptr);
+            bool ok = nasalSys->createModule(moduleName.c_str(), moduleName.c_str(),
+                                             loadScript.c_str(), loadScript.size(),
+                                             nullptr);
+
+            if (!ok) {
+                // TODO: get the Nasal errors logged properly
+                simgear::reportFailure(simgear::LoadFailure::BadData, simgear::ErrorCode::ScenarioLoad,
+                                       "Failed to parse scenario Nasal");
+            }
         }
     }
 
@@ -213,8 +221,7 @@ SGPropertyNode_ptr FGAIManager::registerScenarioFile(SGPropertyNode_ptr root, co
     auto scenariosNode = root->getNode("/sim/ai/scenarios", true);
     SGPropertyNode_ptr sNode;
 
-    // don't report XML errors while loading scenarios, to Sentry
-    flightgear::sentryThreadReportXMLErrors(false);
+    simgear::ErrorReportContext ectx("scenario-path", xmlPath.utf8Str());
 
     try {
         SGPropertyNode_ptr scenarioProps(new SGPropertyNode);
@@ -246,12 +253,14 @@ SGPropertyNode_ptr FGAIManager::registerScenarioFile(SGPropertyNode_ptr root, co
             
             FGAICarrier::extractCarriersFromScenario(xs, sNode);
         } // of scenarios in the XML file
-    } catch (std::exception&) {
+    } catch (sg_exception& e) {
         SG_LOG(SG_AI, SG_WARN, "Skipping malformed scenario file:" << xmlPath);
+        simgear::reportFailure(simgear::LoadFailure::BadData, simgear::ErrorCode::ScenarioLoad,
+                               string{"The scenario couldn't be loaded:"} + e.getFormattedMessage(),
+                               e.getLocation());
         sNode.reset();
     }
 
-    flightgear::sentryThreadReportXMLErrors(true);
     return sNode;
 }
 
@@ -400,7 +409,7 @@ FGAIManager::updateLOD(SGPropertyNode* node)
 }
 
 void
-FGAIManager::attach(FGAIBase *model)
+FGAIManager::attach(const SGSharedPtr<FGAIBase> &model)
 {
     const char* typeString = model->getTypeString();
     SGPropertyNode* root = globals->get_props()->getNode("ai/models", true);
@@ -590,13 +599,18 @@ FGAIBasePtr FGAIManager::getObjectFromProperty(const SGPropertyNode* aProp) cons
 bool
 FGAIManager::loadScenario( const string &id )
 {
-    SGPropertyNode_ptr file = loadScenarioFile(id);
+    SGPath path;
+    SGPropertyNode_ptr file = loadScenarioFile(id, path);
     if (!file) {
         return false;
     }
 
+    simgear::ErrorReportContext ec("scenario-path", path.utf8Str());
     SGPropertyNode_ptr scNode = file->getChild("scenario");
     if (!scNode) {
+        simgear::reportFailure(simgear::LoadFailure::Misconfigured,
+                               simgear::ErrorCode::ScenarioLoad,
+                               "No <scenario> element in file", path);
         return false;
     }
 
@@ -642,14 +656,16 @@ FGAIManager::unloadAllScenarios()
 
 
 SGPropertyNode_ptr
-FGAIManager::loadScenarioFile(const std::string& scenarioName)
+FGAIManager::loadScenarioFile(const std::string& scenarioName, SGPath& outPath)
 {
     auto s = fgGetNode("/sim/ai/scenarios");
     if (!s) return {};
-    
+
     for (auto n : s->getChildren("scenario")) {
         if (n->getStringValue("id") == scenarioName) {
             SGPath path{n->getStringValue("path")};
+            outPath = path;
+            simgear::ErrorReportContext ec("scenario-path", path.utf8Str());
             try {
                 SGPropertyNode_ptr root = new SGPropertyNode;
                 readProperties(path, root);
@@ -657,6 +673,9 @@ FGAIManager::loadScenarioFile(const std::string& scenarioName)
             } catch (const sg_exception &t) {
                 SG_LOG(SG_AI, SG_ALERT, "Failed to load scenario '"
                        << path << "': " << t.getFormattedMessage());
+                simgear::reportFailure(simgear::LoadFailure::BadData, simgear::ErrorCode::ScenarioLoad,
+                                       "Failed to laod scenario XML:" + t.getFormattedMessage(),
+                                       t.getLocation());
             }
         }
     }
