@@ -24,8 +24,6 @@
 #  include "config.h"
 #endif
 
-#include <iostream>
-
 #include <Airports/dynamics.hxx>
 #include <Airports/airportdynamicsmanager.hxx>
 #include <Airports/airport.hxx>
@@ -43,6 +41,9 @@
 
 using std::string;
 
+/**
+Constructer, initializes values to private boolean and FGATCController instances
+*/
 FGATCManager::FGATCManager() :
     controller(NULL),
     prevController(NULL),
@@ -51,53 +52,68 @@ FGATCManager::FGATCManager() :
 {
 }
 
+/**
+Default destructor
+*/
 FGATCManager::~FGATCManager() {
 }
 
+/**
+Sets up ATC subsystem parts depending on other subsystems
+Override of SGSubsystem::postinit()
+Will set private boolean flag "initSucceeded" to true upon conclusion
+*/
 void FGATCManager::postinit()
 {
     int leg = 0;
 
     trans_num = globals->get_props()->getNode("/sim/atc/transmission-num", true);
 
-    // find a reasonable controller for our user's aircraft..
-    // Let's start by working out the following three scenarios: 
-    // Starting on ground at a parking position
-    // Starting on ground at the runway.
-    // Starting in the Air
+    // Assign a controller to the user's aircraft.
+    // Three scenarios are considered:
+    //  - Starting on ground at a parking position
+    //  - Starting on ground at the runway.
+    //  - Starting in the air
     bool onGround  = fgGetBool("/sim/presets/onground");
     string runway  = fgGetString("/sim/atc/runway");
-    string airport = fgGetString("/sim/presets/airport-id");
+    string curAirport = fgGetString("/sim/presets/airport-id");
     string parking = fgGetString("/sim/presets/parkpos");
+    destination = fgGetString("/autopilot/route-manager/destination/airport");
     
     FGAIManager* aiManager = globals->get_subsystem<FGAIManager>();
-    FGAIAircraft* userAircraft = aiManager->getUserAircraft();
+    auto userAircraft = aiManager->getUserAircraft();
+    string callsign = userAircraft->getCallSign();
+    
+    double aircraftRadius = 40; // note that this is currently hardcoded to a one-size-fits all JumboJet value. Should change later.
+    
+    // In case a destination is not set yet, make it equal to the current airport
+    if (destination == "") {
+        destination = curAirport;
+    }
 
-    double aircraftRadius = 40; // note that this is currently hardcoded to a one-size-fits all JumboJet value. Should change later;
-
-    // NEXT UP: Create a traffic Schedule and fill that with appropriate information. This we can use to flight planning.
+    // NEXT UP: Create a traffic schedule and fill that with appropriate information. This we can use for flight planning.
     // Note that these are currently only defaults. 
-    FGAISchedule *trafficRef = new FGAISchedule;
-    trafficRef->setFlightType("gate");
+	userAircraftTrafficRef.reset(new FGAISchedule);
+    userAircraftTrafficRef->setFlightType("gate");
 
-    FGScheduledFlight *flight =  new FGScheduledFlight;
-    flight->setDepartureAirport(airport);
-    flight->setArrivalAirport(airport);
-    flight->initializeAirports();
-    flight->setFlightRules("IFR");
-    flight->setCallSign(userAircraft->getCallSign());
+	userAircraftScheduledFlight.reset(new FGScheduledFlight);
+	userAircraftScheduledFlight->setDepartureAirport(curAirport);
+	userAircraftScheduledFlight->setArrivalAirport(destination);
+	userAircraftScheduledFlight->initializeAirports();
+	userAircraftScheduledFlight->setFlightRules("IFR");
+	userAircraftScheduledFlight->setCallSign(callsign);
     
-    trafficRef->assign(flight);
+    userAircraftTrafficRef->assign(userAircraftScheduledFlight.get());
     std::unique_ptr<FGAIFlightPlan> fp ;
-    userAircraft->setTrafficRef(trafficRef);
+    userAircraft->setTrafficRef(userAircraftTrafficRef.get());
     
-    string flightPlanName = airport + "-" + airport + ".xml";
+    string flightPlanName = curAirport + "-" + destination + ".xml";
     //double cruiseAlt = 100; // Doesn't really matter right now.
     //double courseToDest = 180; // Just use something neutral; this value might affect the runway that is used though...
     //time_t deptime = 0;        // just make sure how flightplan processing is affected by this...
 
 
-    FGAirportDynamicsRef dcs(flightgear::AirportDynamicsManager::find(airport));
+    FGAirportDynamicsRef dcs(flightgear::AirportDynamicsManager::find(curAirport));
     if (dcs && onGround) {// && !runway.empty()) {
 
         ParkingAssignment pk;
@@ -122,8 +138,8 @@ void FGATCManager::postinit()
                     fgSetString("/sim/presets/parkpos", pkl.front()->getName());
                 }
             }
-        } else {
-            pk = dcs->getParkingByName(parking);
+        } else if (!parking.empty()) {
+            pk = dcs->getAvailableParkingByName(parking);
         }
       
         if (pk.isValid()) {
@@ -133,7 +149,7 @@ void FGATCManager::postinit()
             int stationFreq = dcs->getGroundFrequency(1);
             if (stationFreq > 0)
             {
-                //cerr << "Setting radio frequency to : " << stationFreq << endl;
+                SG_LOG(SG_ATC, SG_DEBUG, "Setting radio frequency to : " << stationFreq);
                 fgSetDouble("/instrumentation/comm[0]/frequencies/selected-mhz", ((double) stationFreq / 100.0));
             }
             leg = 1;
@@ -164,7 +180,7 @@ void FGATCManager::postinit()
             int stationFreq = dcs->getTowerFrequency(2);
             if (stationFreq > 0)
             {
-                //cerr << "Setting radio frequency to in airfrequency: " << stationFreq << endl;
+                SG_LOG(SG_ATC, SG_DEBUG, "Setting radio frequency to inair frequency : " << stationFreq);
                 fgSetDouble("/instrumentation/comm[0]/frequencies/selected-mhz", ((double) stationFreq / 100.0));
             }
             fp.reset(new FGAIFlightPlan);
@@ -175,9 +191,20 @@ void FGATCManager::postinit()
             userAircraft->setTakeOffStatus(2);
         } else {
             // We're on the ground somewhere. Handle this case later.
+
+            // important : we are on the ground, so reset the AIFlightPlan back to
+            // a dummy one. Otherwise, in the reposition case, we end up with a
+            // stale flight-plan which confuses other code (eg, PositionInit::finalizeForParking)
+            // see unit test: PosInitTests::testRepositionAtOccupied
+            fp.reset(FGAIFlightPlan::createDummyUserPlan());
+            userAircraft->FGAIBase::setFlightPlan(std::move(fp));
+            controller = nullptr;
+            
+            initSucceeded = true; // should be false?
+            return;
         }
-        
-        if (fp) {
+
+        if (fp && !fp->empty()) {
             fp->getLastWaypoint()->setName( fp->getLastWaypoint()->getName() + string("legend"));
         }
      } else {
@@ -185,15 +212,20 @@ void FGATCManager::postinit()
      }
 
     // Create an initial flightplan and assign it to the ai_ac. We won't use this flightplan, but it is necessary to
-    // keep the ATC code happy. 
+    // keep the ATC code happy.
+    // note in the reposition case, 'fp' is only the new FlightPlan; if we didn't create one here.
+    // we will continue using the existing flight plan (and not restart it, for example)
     if (fp) {
         fp->restart();
         fp->setLeg(leg);
         userAircraft->FGAIBase::setFlightPlan(std::move(fp));
     }
+    
     if (controller) {
         FGAIFlightPlan* plan = userAircraft->GetFlightPlan();
-        controller->announcePosition(userAircraft->getID(), plan, plan->getCurrentWaypoint()->getRouteIndex(),
+        const int routeIndex = (plan && plan->getCurrentWaypoint()) ? plan->getCurrentWaypoint()->getRouteIndex() : 0;
+        controller->announcePosition(userAircraft->getID(), plan,
+                                     routeIndex,
                                      userAircraft->_getLatitude(),
                                      userAircraft->_getLongitude(),
                                      userAircraft->_getHeading(),
@@ -204,15 +236,49 @@ void FGATCManager::postinit()
     initSucceeded = true;
 }
 
+/**
+Shutdown method
+Clears activeStations vector in preparation for clean shutdown
+Override of SGSubsystem::shutdown()
+*/
 void FGATCManager::shutdown()
 {
     activeStations.clear();
+    userAircraftTrafficRef.reset();
+    userAircraftScheduledFlight.reset();
 }
 
+void FGATCManager::reposition()
+{
+    prevController = controller = nullptr;
+    
+// remove any parking assignment form the user flight-plan, so it's
+// available again. postinit() will recompute a new value if required
+    FGAIManager* aiManager = globals->get_subsystem<FGAIManager>();
+    auto userAircraft = aiManager->getUserAircraft();
+    if (userAircraft) {
+        if (userAircraft->GetFlightPlan()) {
+              auto userAIFP = userAircraft->GetFlightPlan();
+              userAIFP->setGate({}); // clear any assignment
+          }
+
+        userAircraft->clearATCController();
+    }
+
+    postinit(); // critical for position-init logic
+}
+
+/**
+Adds FGATCController instance to std::vector activeStations.
+FGATCController is a basic class for every controller
+*/
 void FGATCManager::addController(FGATCController *controller) {
     activeStations.push_back(controller);
 }
 
+/**
+Searches for and removes FGATCController instance from std::vector activeStations
+*/
 void FGATCManager::removeController(FGATCController *controller)
 {
     AtcVecIterator it;
@@ -222,74 +288,117 @@ void FGATCManager::removeController(FGATCController *controller)
     }
 }
 
+/**
+Update the subsystem.
+FlightGear invokes this method every time the subsystem should
+update its state.
+
+@param time The delta time, in seconds, since the last
+update.  On first update, delta time will be 0.
+*/
 void FGATCManager::update ( double time ) {
-    //cerr << "ATC update code is running at time: " << time << endl;
-    // Test code: let my virtual co-pilot handle ATC:
-   
+    SG_LOG(SG_ATC, SG_BULK, "ATC update code is running at time: " << time);
+    
+    // Test code: let my virtual co-pilot handle ATC
     FGAIManager* aiManager = globals->get_subsystem<FGAIManager>();
-    FGAIAircraft* ai_ac = aiManager->getUserAircraft();
-    FGAIFlightPlan *fp = ai_ac->GetFlightPlan();
+    FGAIAircraft* user_ai_ac = aiManager->getUserAircraft();
+    FGAIFlightPlan *fp = user_ai_ac->GetFlightPlan();
+    
+    // Update destination
+    string result = fgGetString("/autopilot/route-manager/destination/airport");
+    if (destination != result && result != "") {
+        destination = result;
+        userAircraftScheduledFlight->setArrivalAirport(destination);
+		userAircraftScheduledFlight->initializeAirports();
+		userAircraftTrafficRef->clearAllFlights();
+		userAircraftTrafficRef->assign(userAircraftScheduledFlight.get());
         
+        auto userAircraft = aiManager->getUserAircraft();
+        userAircraft->setTrafficRef(userAircraftTrafficRef.get());
+    }
+
     /* test code : find out how the routing develops */
     if (fp) {
         int size = fp->getNrOfWayPoints();
-        //cerr << "Setting pos" << pos << " ";
-        //cerr << "setting intentions " ;
+        //SG_LOG(SG_ATC, SG_DEBUG, "Setting pos" << pos << " ");
+        //SG_LOG(SG_ATC, SG_DEBUG, "Setting intentions");
         // This indicates that we have run out of waypoints: Im future versions, the
         // user should be able to select a new route, but for now just shut down the
         // system. 
         if (size < 3) {
-            //cerr << "Shutting down the atc_mgr" << endl;
             return;
         }
 #if 0
         // Test code: Print how far we're progressing along the taxi route. 
-        //std::cerr << "Size of waypoint cue " << size << " ";
+        SG_LOG(SG_ATC, SG_DEBUG, "Size of waypoint cue " << size);
         for (int i = 0; i < size; i++) {
             int val = fp->getRouteIndex(i);
-            //std::cerr << fp->getWayPoint(i)->getName() << " ";
+            SG_LOG(SG_ATC, SG_BULK, fp->getWayPoint(i)->getName() << " ");
             //if ((val) && (val != pos)) {
             //    intentions.push_back(val);
-            //    std::cerr << "[done ] " << std::endl;
+            SG_LOG(SG_ATC, SG_BULK, "[done ]");
             //}
         }
-        //std::cerr << "[done ] " << std::endl;
+        SG_LOG(SG_ATC, SG_BULK, "[done ]");
 #endif
     }
     if (fp) {
-        //cerr << "Currently at leg : " << fp->getLeg() << endl;
+        SG_LOG(SG_ATC, SG_DEBUG, "Currently at leg : " << fp->getLeg());
     }
     
-    controller = ai_ac->getATCController();
+    // Call getATCController method; returns what FGATCController presently controls the user aircraft
+    // - e.g. FGStartupController
+    controller = user_ai_ac->getATCController();
+
+    // Update the ATC dialog
     FGATCDialogNew::instance()->update(time);
+
+    // Controller manager - if controller is set, then will update controller
     if (controller) {
-       //cerr << "name of previous waypoint : " << fp->getPreviousWaypoint()->getName() << endl;
+        SG_LOG(SG_ATC, SG_DEBUG, "name of previous waypoint : " << fp->getPreviousWaypoint()->getName());
+        SG_LOG(SG_ATC, SG_DEBUG, "Currently under control of " << controller->getName());
+
+        // update aircraft information (simulates transponder)
+
+        controller->updateAircraftInformation(user_ai_ac->getID(),
+                                              user_ai_ac->_getLatitude(),
+                                              user_ai_ac->_getLongitude(),
+                                              user_ai_ac->_getHeading(),
+                                              user_ai_ac->getSpeed(),
+                                              user_ai_ac->getAltitude(), time);
 
-        //cerr << "Running FGATCManager::update()" << endl;
-        //cerr << "Currently under control of " << controller->getName() << endl;
-        controller->updateAircraftInformation(ai_ac->getID(),
-                                              ai_ac->_getLatitude(),
-                                              ai_ac->_getLongitude(),
-                                              ai_ac->_getHeading(),
-                                              ai_ac->getSpeed(),
-                                              ai_ac->getAltitude(), time);
         //string airport = fgGetString("/sim/presets/airport-id");
         //FGAirport *apt = FGAirport::findByIdent(airport); 
         // AT this stage we should update the flightplan, so that waypoint incrementing is conducted as well as leg loading. 
+
+        // Ground network visibility:
+        // a) check to see if the message to toggle visibility was called
+        // b) if so, toggle network visibility and reset the transmission
+        // c) therafter disable rendering for the old controller (TODO: should this be earlier?)
+        // d) and render if enabled for the new controller
         int n = trans_num->getIntValue();
+
         if (n == 1) {
-            //cerr << "Toggling ground network visibility " << networkVisible << endl;
+            SG_LOG(SG_ATC, SG_DEBUG, "Toggling ground network visibility " << networkVisible);
             networkVisible = !networkVisible;
             trans_num->setIntValue(-1);
         }
+        
+        // stop rendering the old controller's groundnetwork
         if ((controller != prevController) && (prevController)) {
             prevController->render(false);
         }
+
+        // render the path for the present controller if the ground network is set to visible
         controller->render(networkVisible);
 
-        //cerr << "Adding groundnetWork to the scenegraph::update" << endl;
+        SG_LOG(SG_ATC, SG_BULK, "Adding ground network to the scenegraph::update");
+
+        // reset previous controller for next update() iteration
         prevController = controller;
    }
+
+   // update the active ATC stations
    for (AtcVecIterator atc = activeStations.begin(); atc != activeStations.end(); atc++) {
        (*atc)->update(time);
    }
