@@ -18,15 +18,15 @@
 //
 // $Id$
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
+#include <config.h>
 
 #include <algorithm>
 #include <string>
 #include <vector>
 
 #include <simgear/compiler.h>
+#include <simgear/structure/SGWeakPtr.hxx>
+#include <simgear/structure/exception.hxx>
 
 #include <Environment/environment_mgr.hxx>
 #include <Environment/environment.hxx>
@@ -68,7 +68,10 @@ public:
 
   ~ParkingAssignmentPrivate()
   {
-    dynamics->releaseParking(parking);
+      FGAirportDynamicsRef strongRef = dynamics.lock();
+      if (strongRef) {
+          strongRef->releaseParking(parking);
+      }
   }
 
   void release()
@@ -85,7 +88,10 @@ public:
 
   unsigned int refCount;
   FGParkingRef parking;
-  FGAirportDynamicsRef dynamics;
+    
+// we don't want an owning ref here, otherwise we
+// have a circular ownership from AirportDynamics -> us
+  SGWeakPtr<FGAirportDynamics> dynamics;
 };
 
 ParkingAssignment::ParkingAssignment() :
@@ -324,6 +330,22 @@ ParkingAssignment FGAirportDynamics::getParkingByName(const std::string& name) c
     return ParkingAssignment();
 }
 
+ParkingAssignment FGAirportDynamics::getAvailableParkingByName(const std::string & name)
+{
+    const FGParkingList& parkings(parent()->groundNetwork()->allParkings());
+    auto it = std::find_if(parkings.begin(), parkings.end(), [this, name] (FGParkingRef parking){
+       if (parking->name() != name)
+           return false;
+        
+        return this->isParkingAvailable(parking);
+    });
+    
+    if (it == parkings.end())
+        return {}; // no assignment possible
+    
+    return {*it, this};
+}
+
 FGParkingRef FGAirportDynamics::getOccupiedParkingByName(const std::string& name) const
 {
     auto it = std::find_if(occupiedParkings.begin(), occupiedParkings.end(),
@@ -781,15 +803,16 @@ bool FGAirportDynamics::innerGetActiveRunway(const string & trafficType,
 
     if (action == 2)            // landing
     {
-        int nr = landing.size();
-        if (nr) {
+        if (!landing.empty()) {
             runway = chooseRwyByHeading(landing, heading);
-        } else {                //fallback
+        }
+        
+        if (runway.empty()) { //fallback
             runway = chooseRunwayFallback();
         }
     }
 
-    return true;
+    return !runway.empty();
 }
 
 string FGAirportDynamics::chooseRwyByHeading(stringVec rwys,
@@ -824,15 +847,29 @@ void FGAirportDynamics::getActiveRunway(const string & trafficType,
                                         double heading)
 {
     bool ok = innerGetActiveRunway(trafficType, action, runway, heading);
-    if (!ok) {
+    if (!ok || runway.empty()) {
         runway = chooseRunwayFallback();
     }
 }
 
 string FGAirportDynamics::chooseRunwayFallback()
 {
+    if (_ap->numRunways() == 0) {
+        SG_LOG(SG_AI, SG_WARN, "FGAirportDynamics: asked for runway at probable heliport, check for a bogus schedule referencing:" << _ap->ident());
+
+        // by throwing, we should cause the AIManager to shut down this flight, which
+        // is what need; the calling code assumes AITraffic has a valid runway, and
+        // we just don't
+        throw sg_exception("No runways available at airport", _ap->ident());
+    }
+
     FGRunway *rwy = _ap->getActiveRunwayForUsage();
-    return rwy ? rwy->ident() : std::string();
+    if (!rwy) {
+        SG_LOG(SG_AI, SG_WARN, "FGAirportDynamics::chooseRunwayFallback failed at " << _ap->ident()); 
+        // let's use runway 0
+        return _ap->getRunwayByIndex(0)->ident();
+    }
+    return rwy->ident();
 }
 
 double FGAirportDynamics::getElevation() const
