@@ -7,21 +7,21 @@
  ------------- Copyright (C) 2000 Anthony K. Peden -------------
 
  This program is free software; you can redistribute it and/or modify it under
- the terms of the GNU Lesser General Public License as published by the Free Software
- Foundation; either version 2 of the License, or (at your option) any later
- version.
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option) any
+ later version.
 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.
 
- You should have received a copy of the GNU Lesser General Public License along with
- this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Lesser General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 59
+ Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
- Further information about the GNU Lesser General Public License can also be found on
- the world wide web at http://www.gnu.org.
+ Further information about the GNU Lesser General Public License can also be
+ found on the world wide web at http://www.gnu.org.
 
 FUNCTIONAL DESCRIPTION
 --------------------------------------------------------------------------------
@@ -40,8 +40,6 @@ INCLUDES
 #include "FGKinemat.h"
 #include "input_output/FGXMLElement.h"
 #include "models/FGFCS.h"
-#include <iostream>
-#include <cstdlib>
 
 using namespace std;
 
@@ -51,7 +49,8 @@ namespace JSBSim {
 CLASS IMPLEMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-FGKinemat::FGKinemat(FGFCS* fcs, Element* element) : FGFCSComponent(fcs, element)
+FGKinemat::FGKinemat(FGFCS* fcs, Element* element)
+  : FGFCSComponent(fcs, element)
 {
   Element *traverse_element, *setting_element;
   double tmpDetent;
@@ -81,7 +80,7 @@ FGKinemat::FGKinemat(FGFCS* fcs, Element* element) : FGFCSComponent(fcs, element
     exit(-1);
   }
 
-  FGFCSComponent::bind();
+  bind(element);
 
   Debug(0);
 }
@@ -103,7 +102,8 @@ bool FGKinemat::Run(void )
 
   if (DoScale) Input *= Detents.back();
 
-  if (IsOutput) Output = OutputNodes[0]->getDoubleValue();
+  if (!OutputNodes.empty())
+    Output = OutputNodes[0]->getDoubleValue();
 
   Input = Constrain(Detents.front(), Input, Detents.back());
 
@@ -152,7 +152,7 @@ bool FGKinemat::Run(void )
   }
 
   Clip();
-  if (IsOutput) SetOutput();
+  SetOutput();
 
   return true;
 }
