@@ -24,6 +24,7 @@
 
 // standard
 #include <string>
+#include <dlfcn.h>
 
 // OS
 
@@ -34,6 +35,8 @@
 #include <Foundation/NSLocale.h>
 #include <AppKit/NSAlert.h>
 
+#include <osgViewer/api/Cocoa/GraphicsWindowCocoa>
+
 // simgear
 #include <simgear/misc/sg_path.hxx>
 #include <simgear/debug/logstream.hxx>
@@ -47,6 +50,9 @@
 
 #if defined(HAVE_QT)
 #  include <GUI/QtLauncher.hxx>
+#  include <QOpenGLContext>
+#  include <QtPlatformHeaders/QCocoaNativeContext>
+#  include <QWindow>
 #endif
 
 NSString* stdStringToCocoa(const std::string& s)
@@ -151,10 +157,45 @@ SGPath Options::platformDefaultRoot() const
     dataDir.append("data");
     return dataDir;
 }
+
+#if defined(HAVE_QT)
+QOpenGLContext* qtContextFromOSG(osg::GraphicsContext* context)
+{
+    osgViewer::GraphicsWindowCocoa* win = dynamic_cast<osgViewer::GraphicsWindowCocoa*>(context);
+    if (!win) {
+        return nullptr;
+    }
+    
+    auto cnc = QCocoaNativeContext{win->getContext()};
+    QVariant nativeHandle = QVariant::fromValue(cnc);
+
+    if (nativeHandle.isNull())
+        return nullptr;
+    
+    std::unique_ptr<QOpenGLContext> qtOpenGLContext(new QOpenGLContext);
+    qtOpenGLContext->setNativeHandle(nativeHandle);
+    if (!qtOpenGLContext->create()) {
+        return nullptr;
+    }
+    
+    return qtOpenGLContext.release();
+}
+
+QWindow* qtWindowFromOSG(osgViewer::GraphicsWindow* graphicsWindow)
+{
+    osgViewer::GraphicsWindowCocoa* win = dynamic_cast<osgViewer::GraphicsWindowCocoa*>(graphicsWindow);
+    if (!win) {
+        return nullptr;
+    }
+    
+    NSView* osgView = [win->getContext() view];
+    return QWindow::fromWinId(reinterpret_cast<qlonglong>(osgView));
+}
+#endif
     
 } // of namespace flightgear
 
-string_list FGLocale::getUserLanguage()
+string_list FGLocale::getUserLanguages()
 {
     CocoaAutoreleasePool ap;
     string_list result;
@@ -222,3 +263,32 @@ void cocoaRegisterTerminateHandler()
      name:NSApplicationWillTerminateNotification object:app];
 #endif
 }
+
+
+
+bool cocoaIsRunningTranslocated()
+{
+    // #define NSAppKitVersionNumber10_11 1404
+    if (floor(NSAppKitVersionNumber) <= 1404) {
+        return false;
+    }
+
+    void *handle = dlopen("/System/Library/Frameworks/Security.framework/Security", RTLD_LAZY);
+    if (handle == NULL) {
+        return false;
+    }
+
+    bool isTranslocated = false;
+    using SecIsTranslocatedFunc = Boolean (*)(CFURLRef path, bool *isTranslocated, CFErrorRef * __nullable error);
+
+    SecIsTranslocatedFunc f = reinterpret_cast<SecIsTranslocatedFunc>(dlsym(handle, "SecTranslocateIsTranslocatedURL"));
+    if (f != NULL) {
+        auto url = CFBundleCopyBundleURL(CFBundleGetMainBundle());
+        f(url, &isTranslocated, NULL);
+
+        CFRelease(url);
+    }
+
+    dlclose(handle);
+    return isTranslocated;
+}
