@@ -287,6 +287,9 @@ void LauncherController::saveSettings()
 
 void LauncherController::collectAircraftArgs()
 {
+    if (m_skipAircraftFromArgs)
+        return;
+
     // aircraft
     if (!m_selectedAircraft.isEmpty()) {
         if (m_selectedAircraft.isLocalFile()) {
@@ -339,7 +342,7 @@ void LauncherController::restoreAircraft()
 
 void LauncherController::doRun()
 {
-    flightgear::addSentryBreadcrumb("Launcher: fly!", "info");
+    flightgear::addSentryBreadcrumb("Launcher: fly!:" + m_selectedAircraft.toString().toStdString(), "info");
     flightgear::Options* opt = flightgear::Options::sharedInstance();
     m_config->reset();
     m_config->collect();
@@ -349,6 +352,8 @@ void LauncherController::doRun()
     QVariant locSet = m_location->saveLocation();
     m_locationHistory->insert(locSet);
 
+    flightgear::addSentryBreadcrumb("acft path:" + m_selectedAircraftInfo->pathOnDisk().toStdString(), "info");
+
     // aircraft paths
     QSettings settings;
     QString downloadDir = settings.value("downloadSettings/downloadDir").toString();
@@ -612,6 +617,13 @@ void LauncherController::setEnvironmentSummary(QStringList environmentSummary)
 
 void LauncherController::fly()
 {
+    // avoid duplicate calls to fly, if the user's system is slow, and they
+    // generate multiple clicks / events before the qApp->exit() fires.
+    // without this, we can apply options, etc twice which breaks.
+    if (m_flyRequested)
+        return;
+    m_flyRequested = true;
+
 	if (m_inAppMode) {
 		doApply();
 		m_keepRunningInAppMode = false;
