@@ -123,11 +123,28 @@ bool AircraftItem::initFromFile(QDir dir, QString filePath)
 
     if (sim->hasChild("previews")) {
         SGPropertyNode_ptr previewsNode = sim->getChild("previews");
+        const QString aircraftPrefix = "Aircraft/" + dir.dirName() + "/";
+        
         for (auto previewNode : previewsNode->getChildren("preview")) {
             // add file path as url
             QString pathInXml = QString::fromStdString(previewNode->getStringValue("path"));
+
+            // https://sourceforge.net/p/flightgear/codetickets/2644/
+            // allow Aircraft/ prefixed paths in previews; what we're actually doing here
+            // is trimming them back to the style we expect
+            if (pathInXml.startsWith(aircraftPrefix)) {
+                pathInXml = pathInXml.mid(aircraftPrefix.length());
+            }
+            
             QString previewPath = dir.absoluteFilePath(pathInXml);
+            
+            if (!QFile::exists(previewPath)) {
+                qWarning() << "Missing local preview file" << previewPath;
+                continue;
+            }
+            
             previews.append(QUrl::fromLocalFile(previewPath));
+
         }
     }
 
@@ -403,11 +420,6 @@ protected:
         flightgear::addSentryBreadcrumb("AircraftScan started", "info");
         readCache();
 
-        // avoid filling up Sentry with many reports
-        // from unmaintained aircraft. We'll still fail if soemeone tries
-        // to use the aircraft, but that's 100x less common.
-        flightgear::sentryThreadReportXMLErrors(false);
-
         Q_FOREACH(QString d, m_dirs) {
             const auto p = SGPath::fromUtf8(d.toUtf8().toStdString());
             m_currentScanDir->setCurrentPath(p);
@@ -474,6 +486,10 @@ private:
         QStringList filters;
         filters << "*-set.xml";
         Q_FOREACH(QFileInfo child, path.entryInfoList(QDir::Dirs | QDir::NoDotAndDotDot)) {
+            if (m_done) { // thread termination bail-out
+                return;
+            }
+            
             QDir childDir(child.absoluteFilePath());
             QMap<QString, AircraftItemPtr> baseAircraft;
             QList<AircraftItemPtr> variants;
@@ -718,7 +734,7 @@ void LocalAircraftCache::abandonCurrentScan()
                 &LocalAircraftCache::onScanFinished);
 
         d->m_scanThread->setDone();
-        if (!d->m_scanThread->wait(2000)) {
+        if (!d->m_scanThread->wait(32000)) {
             qWarning() << Q_FUNC_INFO << "scan thread failed to terminate";
         }
         d->m_scanThread.reset();
@@ -808,7 +824,6 @@ LocalAircraftCache::readAircraftProperties(const SGPath &setPath, SGPropertyNode
     dp->setCurrentAircraftPath(setPath);
 
     ParseSetXMLResult result = ParseSetXMLResult::Failed;
-    flightgear::sentryThreadReportXMLErrors(false);
 
     try {
         readProperties(setPath, props);
@@ -820,7 +835,6 @@ LocalAircraftCache::readAircraftProperties(const SGPath &setPath, SGPropertyNode
     }
 
     rm->removeProvider(dp.get());
-    flightgear::sentryThreadReportXMLErrors(true);
     return result;
 }
 
