@@ -131,7 +131,9 @@ void AtisSpeaker::valueChanged(SGPropertyNode * node)
   }
 
   FGSoundManager * smgr = globals->get_subsystem<FGSoundManager>();
-  assert(smgr != NULL);
+  if (!smgr) {
+      return;
+  }
 
   SG_LOG(SG_INSTR, SG_DEBUG,"AtisSpeaker voice is " << voice );
   FLITEVoiceSynthesizer * synthesizer = dynamic_cast<FLITEVoiceSynthesizer*>(smgr->getSynthesizer(voice));
@@ -414,8 +416,9 @@ private:
 			if( channelNum != _channelNum ) _channelNum = channelNum;
 
 			if( _frequency != channel ) {
-				_channel = _frequency; //triggers recursion
-			}
+                const double channelValue = c / 1000.0;
+                _channel = channelValue;
+            }
 		} else {
 			_channelSpacing = 8.33;
 
@@ -582,7 +585,7 @@ void CommRadioImpl::init()
   
   auto soundManager = globals->get_subsystem<SGSoundMgr>();
   if (soundManager) {
-    _sampleGroup = soundManager->find("atc", false);
+    _sampleGroup = soundManager->find("atc", true);
   }
 }
 
@@ -618,11 +621,13 @@ void CommRadioImpl::update(double dt)
   if (_stationTTL <= 0.0) {
     _stationTTL = 30.0;
 
-    // Note:  122.375 must be rounded DOWN to 122370
-    // in order to be consistent with apt.dat et cetera.
-    int freqKhz = 10 * static_cast<int>(_frequency * 100 + 0.25);
-    _commStationForFrequency = flightgear::CommStation::findByFreq(freqKhz, position, NULL);
+    int freqKhz = static_cast<int>(_frequency * 1000 + 0.5);
+    // make sure the frequency is integral multiple of 25, if not using 8.333 kHz spacing
+    if (!_useEightPointThree && freqKhz % 25) {
+        freqKhz += 5;
+    }
 
+    _commStationForFrequency = flightgear::CommStation::findByFreq(freqKhz, position, NULL);
   }
 
   if (false == _commStationForFrequency.valid()) {
