@@ -585,32 +585,36 @@ TCAS::ThreatDetector::update(void)
     tcas->advisoryGenerator.setAlarmThresholds(pAlarmThresholds);
 }
 
+// If plane's transponder is enabled, return true with o_altFt set to
+// altitude. Otherwise return false.
+//
+static bool checkTransponderLocal(const SGPropertyNode* pModel, float velocityKt, float& o_altFt)
+{
+    if (pModel->getBoolValue("controls/invisible", false /*default*/))
+    {
+        // For MP aircraft (name='multiplayer') that are being ignored.
+        return false;
+    }
+    if (pModel->getNameString() == "aircraft")
+    {
+        /* assume all non-MP and non-Swift (i.e. AI) aircraft have their transponder switched off while taxiing/parking
+         * (at low speed) */
+        if (velocityKt < 40.0)  return false;
+        o_altFt = pModel->getDoubleValue("position/altitude-ft");
+        return true;
+    }
+    o_altFt = pModel->getIntValue("instrumentation/transponder/altitude", -9999);
+    // must have Mode C (altitude) transponder to be visible.
+    // "-9999" is a special value used by src/Instrumentation/transponder.cxx to indicate the non-transmission of a value.
+    return (o_altFt != -9999);
+}
+
 /** Check if plane's transponder is enabled. */
 bool
 TCAS::ThreatDetector::checkTransponder(const SGPropertyNode* pModel, float velocityKt)
 {
-    const string name = pModel->getName();
-    if (name != "multiplayer" && name != "aircraft" && name != "swift")
-    {
-        // assume non-MP/non-AI planes (e.g. ships) have no transponder
-        return false;
-    }
-
-    if (velocityKt < 40)
-    {
-        /* assume all pilots have their transponder switched off while taxiing/parking
-         * (at low speed) */
-        return false;
-    }
-
-    if ((name == "multiplayer")&&
-        (pModel->getBoolValue("controls/invisible")))
-    {
-        // ignored MP plane: pretend transponder is switched off
-        return false;
-    }
-
-    return true;
+    float altFt;
+    return checkTransponderLocal(pModel, velocityKt, altFt);
 }
 
 /** Check if plane is a threat. */
@@ -625,11 +629,11 @@ TCAS::ThreatDetector::checkThreat(int mode, const SGPropertyNode* pModel)
 
     float velocityKt  = pModel->getDoubleValue("velocities/true-airspeed-kt");
 
-    if (!checkTransponder(pModel, velocityKt))
+    float altFt;
+    if (!checkTransponderLocal(pModel, velocityKt, altFt))
         return ThreatInvisible;
 
     int threatLevel = ThreatNone;
-    float altFt = pModel->getDoubleValue("position/altitude-ft");
     currentThreat.relativeAltitudeFt = altFt - self.pressureAltFt;
 
     // save computation time: don't care when relative altitude is excessive
