@@ -139,4 +139,12 @@ VerticalSpeedIndicator::update (double dt)
     }
 }
 
+
+// Register the subsystem.
+#if 0
+SGSubsystemMgr::InstancedRegistrant<VerticalSpeedIndicator> registrantVerticalSpeedIndicator(
+    SGSubsystemMgr::FDM,
+    {{"instrumentation", SGSubsystemMgr::Dependency::HARD}});
+#endif
+
 // end of vertical_speed_indicator.cxx
