@@ -154,6 +154,7 @@
 #include "util.hxx"
 #include "AircraftDirVisitorBase.hxx"
 #include <Main/sentryIntegration.hxx>
+#include <Main/ErrorReporter.hxx>
 
 #if defined(SG_MAC)
 #include <GUI/CocoaHelpers.h> // for Mac impl of platformDefaultDataPath()
@@ -237,6 +238,7 @@ public:
         } catch ( const sg_exception &e ) {
             SG_LOG(SG_IO, SG_ALERT,
                    "Error reading aircraft: " << e.getFormattedMessage());
+            flightgear::addSentryBreadcrumb("Aircraft-dir=" + aircraftDir, "error");
             SG_LOG(SG_IO, SG_ALERT, "aircraft dir is:" << aircraftDir);
             flightgear::fatalMessageBoxWithoutExit(
                 "Error reading aircraft",
@@ -246,6 +248,7 @@ public:
         }
 
         checkAircraftMinVersion();
+        checkAircraftDirName();
 
           // apply state after the -set.xml, but before any options are are set
           flightgear::applyInitialState();
@@ -253,11 +256,13 @@ public:
       } else {
         SG_LOG(SG_GENERAL, SG_ALERT, "aircraft '" << _searchAircraft << 
                "' not found in specified dir:" << aircraftDir);
+        flightgear::addSentryBreadcrumb("Aircraft-dir=" + aircraftDir, "error");
         flightgear::fatalMessageBoxWithoutExit(
-          "Aircraft not found",
-          "The requested aircraft (" + aircraft + ") could not be found "
-          "in the specified location.",
-          aircraftDir);
+            "Aircraft not found",
+            "The requested aircraft (" + aircraft + ") could not be found "
+                                                    "in the specified location. (" +
+                aircraftDir + ")",
+            aircraftDir);
         return false;
       }
     }
@@ -281,12 +286,24 @@ public:
       SG_LOG(SG_GENERAL, SG_ALERT,
              "Cannot find the specified aircraft: '" << aircraft << "'");
 
+      flightgear::addSentryBreadcrumb("Aircraft paths: " + SGPath::join(globals->get_aircraft_paths(), ";"), "error");
+
       SG_LOG(SG_GENERAL, SG_ALERT, "\tin paths:" << SGPath::join(globals->get_aircraft_paths(), ";"));
 
+      std::string notFoundMessage;
+      bool reportToSentry = true;
+      if (globals->get_aircraft_paths().empty()) {
+          notFoundMessage = "The requested aircraft (" + aircraft + ") could not be found. No aircraft paths are configured.";
+          reportToSentry = false; // no need to log these
+      } else {
+          notFoundMessage = "The requested aircraft (" + aircraft + ") could not be found in any of the search paths.";
+      }
+
       flightgear::fatalMessageBoxWithoutExit(
-        "Aircraft not found",
-        "The requested aircraft (" + aircraft + ") could not be found "
-        "in any of the search paths.");
+          "Aircraft not found",
+          notFoundMessage,
+          {},
+          reportToSentry);
       return false;
     }
     
@@ -296,9 +313,10 @@ public:
       SG_LOG(SG_GENERAL, SG_ALERT, "Unable to find -set file:" << _foundPath);
       return false;
     }
-    
+
+
     try {
-      readProperties(_foundPath, globals->get_props());
+        readProperties(_foundPath, globals->get_props());
     } catch ( const sg_exception &e ) {
       SG_LOG(SG_INPUT, SG_ALERT,
              "Error reading aircraft: " << e.getFormattedMessage());
@@ -313,6 +331,7 @@ public:
       flightgear::applyInitialState();
 
       checkAircraftMinVersion();
+      checkAircraftDirName();
 
     return true;
   }
@@ -423,7 +442,34 @@ private:
       
       return true;
   }
-  
+
+  bool checkAircraftDirName()
+  {
+      auto expectedDirNode = globals->get_props()->getNode("/sim/expected-aircraft-dir-name");
+      const string aircraftId = fgGetString("/sim/aircraft");
+      if (aircraftId != fgGetString("/sim/aircraft-id")){
+          // Skip the check for aircraft installed from catalog.
+          return true;
+      }
+
+      if (expectedDirNode) {
+          const string expectedDir = expectedDirNode->getStringValue();
+          const SGPath dir(fgGetString("/sim/aircraft-dir"));
+          if (dir.file() != expectedDirNode->getStringValue()) {
+              flightgear::fatalMessageBoxThenExit("Aircraft folder named incorrectly",
+                                          "The folder of the selected aircraft must be named '" + expectedDir +
+                                          "' (instead of '" + dir.file() + "') to work correctly. If you downloaded it yourself, " +
+                                          "please ensure the folder is called '" +
+                                          expectedDir + "' and re-name if necessary.");
+              return false;
+          }
+      } else {
+          // TODO Uncomment for the next release.
+          //SG_LOG(SG_AIRCRAFT, SG_DEV_ALERT, "Aircraft does not specify the required aircraft directory name: please add one at /sim/expected-aircraft-dir-name");
+      }
+      return true;
+  }
+
   std::string _searchAircraft;
   SGPath _foundPath;
   SGPropertyNode* _cache;
@@ -638,7 +684,7 @@ int fgInitConfig ( int argc, char **argv, bool reinit )
     createBaseStorageDirForAddons(exportDir.path());
 
     // Set /sim/fg-home.  Use FG_HOME if necessary.
-    // deliberately not a tied property, for fgValidatePath security
+    // deliberately not a tied property, for SGPath::validate() security
     // write-protect to avoid accidents
     SGPropertyNode *home = fgGetNode("/sim", true);
     home->removeChild("fg-home", 0);
@@ -708,7 +754,7 @@ int fgInitConfig ( int argc, char **argv, bool reinit )
 
 static void initAircraftDirsNasalSecurity()
 {
-    // deliberately not a tied property, for fgValidatePath security
+    // deliberately not a tied property, for SGPath::validate() security
     // write-protect to avoid accidents
     SGPropertyNode* sim = fgGetNode("/sim", true);
     sim->removeChildren("fg-aircraft");
@@ -749,10 +795,14 @@ int fgInitAircraft(bool reinit)
     
     FindAndCacheAircraft f(globals->get_props());
     const bool haveExplicit = f.haveExplicitAircraft();
+    if (haveExplicit) {
+        flightgear::addSentryBreadcrumb("Have explicit aircraft-dr and -set.xml", "info");
+    }
 
     SGSharedPtr<Root> pkgRoot(globals->packageRoot());
     SGPropertyNode* aircraftProp = fgGetNode("/sim/aircraft", true);
-    string aircraftId(aircraftProp->getStringValue());
+    const string fullyQualifiedAircraftId = fgGetString("/sim/aircraft-id");
+    string aircraftId = fullyQualifiedAircraftId.empty() ? aircraftProp->getStringValue() : fullyQualifiedAircraftId;
 
     flightgear::addSentryTag("aircraft", aircraftId);
         
@@ -763,7 +813,14 @@ int fgInitAircraft(bool reinit)
 
     if (acftPackage) {
         if (acftPackage->isInstalled()) {
+            flightgear::addSentryBreadcrumb("Loading aircraft from installed package", "info");
             SG_LOG(SG_GENERAL, SG_INFO, "Loading aircraft from package:" << acftPackage->qualifiedId());
+            
+            // if we resolved a non-qualified ID, set the full one back to /sim/aircraft-id
+            fgSetString("/sim/aircraft-id", acftPackage->qualifiedId());
+
+            // replace this tag, so we know which hangar is in use
+            flightgear::addSentryTag("aircraft", acftPackage->qualifiedId());
 
             // set catalog path so intra-package dependencies within the catalog
             // are resolved correctly.
@@ -773,6 +830,8 @@ int fgInitAircraft(bool reinit)
             InstallRef acftInstall = acftPackage->install();
             fgSetString("/sim/aircraft-dir", acftInstall->path().utf8Str());
 
+            flightgear::addSentryBreadcrumb("Computed aircraft-dir as:" + acftInstall->path().utf8Str(), "info");
+
             // overwrite the fully qualified ID with the aircraft one, so the
             // code in FindAndCacheAircraft works as normal
             // note since we may be using a variant, we can't use the package ID
@@ -784,6 +843,7 @@ int fgInitAircraft(bool reinit)
 
             // run the traditional-code path below
         } else {
+            flightgear::addSentryBreadcrumb("Package is not installed:" + aircraftId, "info");
 #if 0
             // naturally the better option would be to on-demand install it!
             flightgear::fatalMessageBoxWithoutExit(
@@ -1244,7 +1304,7 @@ void fgStartReposition()
 
 void fgStartNewReset()
 {
-    flightgear::addSentryTag("have-reset", "yes");
+    flightgear::updateSentryTag("have-reset", "yes");
 
     // save user settings now, so that USERARCIVE-d values changes since the
     // last init are recorded and hence re-loaded when we fgInitConfig down
@@ -1259,6 +1319,20 @@ void fgStartNewReset()
     fgSetBool("/sim/signals/reinit", true);
     fgSetBool("/sim/freeze/master", true);
     
+    // pause the osgDB requests right now, but more may appear; we will
+    // clear and cancel further down once shutdown has occured
+    FGRenderer* render = globals->get_renderer();
+    auto pager = render->getViewer()->getDatabasePager();
+    pager->setAcceptNewDatabaseRequests(false);
+    pager->cancel();
+    
+    // extra clear is needed to ensure compile/merge lists are also empty
+    pager->clear();
+    
+    assert(pager->getDataToMergeListSize() == 0);
+    assert(pager->getDataToCompileListSize() == 0);
+
+    
     SGSubsystemMgr* subsystemManger = globals->get_subsystem_mgr();
     // Nasal is added in fgPostInit, ensure it's already shutdown
     // before other subsystems, so Nasal listeners don't fire during shutdown
@@ -1277,7 +1351,10 @@ void fgStartNewReset()
         SGSubsystemGroup* grp = subsystemManger->get_group(static_cast<SGSubsystemMgr::GroupType>(g));
         for (auto nm : grp->member_names()) {
             if ((nm == "time") || (nm == "terrasync") || (nm == "events")
-                || (nm == "lighting") || (nm == FGScenery::staticSubsystemClassId()))
+                || (nm == "lighting") 
+                || (nm == FGScenery::staticSubsystemClassId())
+                || (nm == flightgear::ErrorReporter::staticSubsystemClassId())
+                )
             {
                 continue;
             }
@@ -1297,7 +1374,6 @@ void fgStartNewReset()
     // drop any references to AI objects with TACAN
     flightgear::NavDataCache::instance()->clearDynamicPositioneds();
 
-    FGRenderer* render = globals->get_renderer();
     // needed or we crash in multi-threaded OSG mode
     render->getViewer()->stopThreading();
     
@@ -1308,11 +1384,6 @@ void fgStartNewReset()
     FGScenery::getPagerSingleton()->clearRequests();
     flightgear::CameraGroup::setDefault(NULL);
     
-    // don't cancel the pager until after shutdown, since AIModels (and
-    // potentially others) can queue delete requests on the pager.
-    render->getViewer()->getDatabasePager()->cancel();
-    render->getViewer()->getDatabasePager()->clear();
-    
     osgDB::Registry::instance()->clearObjectCache();
     // Pager requests depend on this, so don't clear it until now
     sgUserDataInit( NULL );
@@ -1325,7 +1396,7 @@ void fgStartNewReset()
 
     globals->set_renderer(NULL);
     globals->set_matlib(NULL);
-
+    
     flightgear::unregisterMainLoopProperties();
     FGReplayData::resetStatisticsProperties();
 
@@ -1396,8 +1467,9 @@ void fgStartNewReset()
 
     sgUserDataInit( globals->get_props() );
 
-    viewer->getDatabasePager()->setUpThreads(20, 1);
-    
+    viewer->getDatabasePager()->setUpThreads(2, 1);
+    viewer->getDatabasePager()->setAcceptNewDatabaseRequests(true);
+
     // must do this before preinit for Rembrandthe
     flightgear::CameraGroup::buildDefaultGroup(viewer.get());
     render->preinit();
@@ -1432,16 +1504,11 @@ void fgInitPackageRoot()
         return;
     }
 
-    SGPath packageAircraftDir = flightgear::Options::sharedInstance()->valueForOption("download-dir");
-    if (packageAircraftDir.isNull()) {
-        packageAircraftDir = flightgear::defaultDownloadDir();
-    }
-
+    SGPath packageAircraftDir = flightgear::Options::sharedInstance()->actualDownloadDir();
     packageAircraftDir.append("Aircraft");
 
     SG_LOG(SG_GENERAL, SG_INFO, "init package root at:" << packageAircraftDir);
 
-
     SGSharedPtr<Root> pkgRoot(new Root(packageAircraftDir, FLIGHTGEAR_VERSION));
     // set the http client later (too early in startup right now)
     globals->setPackageRoot(pkgRoot);
@@ -1473,7 +1540,7 @@ int fgUninstall()
     if (terrasyncPath.exists()) {
         simgear::Dir dir(terrasyncPath);
         if (!dir.remove(true /*recursive*/)) {
-            fprintf(stderr, "Errors occurred trying to remove Documents/FlightGear/TerraSync");
+            std::cerr << "Errors occurred trying to remove " << terrasyncPath << std::endl;
         }
     }
 
@@ -1481,7 +1548,15 @@ int fgUninstall()
     if (packagesPath.exists()) {
         simgear::Dir dir(packagesPath);
         if (!dir.remove(true /*recursive*/)) {
-            fprintf(stderr, "Errors occurred trying to remove Documents/FlightGear/Aircraft");
+            std::cerr << "Errors occurred trying to remove " << packagesPath << std::endl;
+        }
+    }
+
+    SGPath cachePath = p / "TextureCache";
+    if (cachePath.exists()) {
+        simgear::Dir dir(cachePath);
+        if (!dir.remove(true /*recursive*/)) {
+            std::cerr << "Errors occurred trying to remove " << cachePath << std::endl;
         }
     }
 #endif
