@@ -64,15 +64,15 @@ auto OSG_messageWhitelist = {
 };
 
 auto XML_messageWhitelist = {
-     "Cannot open file",
-     "not well-formed (invalid token)",
-     "mismatched tag",
-     "syntax error",
-     "no element found",
-     "Root element name is",
-     "XML or text declaration not at start of entity",
-     "Failed to open file"
-};
+    "Cannot open file",
+    "not well-formed (invalid token)",
+    "mismatched tag",
+    "syntax error",
+    "no element found",
+    "Root element name is",
+    "XML or text declaration not at start of entity",
+    "Failed to open file",
+    "unclosed token"};
 
 auto exception_messageWhitelist = {
     "position is invalid, NaNs",    ///< avoid spam when NaNs occur
@@ -291,11 +291,9 @@ void initSentry()
     // FLIGHTGEAR_SENTRY_API_KEY
     sentry_options_set_dsn(options, SENTRY_API_KEY);
     
-    if (strcmp(FG_BUILD_TYPE, "Dev") == 0) {
-        sentry_options_set_release(options, "flightgear-dev@" FLIGHTGEAR_VERSION);
-    } else {
-        sentry_options_set_release(options, "flightgear@" FLIGHTGEAR_VERSION);
-    }
+    // modified for 2020.3: we don't want to create '-dev' releases
+    // even if people self-compile the branch.
+    sentry_options_set_release(options, "flightgear@" FLIGHTGEAR_VERSION);
     
     const auto buildString = std::to_string(JENKINS_BUILD_NUMBER);
     sentry_options_set_dist(options, buildString.c_str());
@@ -408,6 +406,18 @@ void addSentryTag(const char* tag, const char* value)
     sentry_set_tag(tag, value);
 }
 
+void updateSentryTag(const std::string& tag, const std::string& value)
+{
+    if (tag.empty() || value.empty())
+        return;
+
+    if (!static_sentryEnabled)
+        return;
+
+    sentry_remove_tag(tag.c_str());
+    sentry_set_tag(tag.c_str(), value.c_str());
+}
+
 void sentryReportNasalError(const std::string& msg, const string_list& stack)
 {
     if (!static_sentryEnabled)
@@ -490,6 +500,25 @@ void sentryThreadReportXMLErrors(bool report)
     perThread_reportXMLParseErrors = report;
 }
 
+void sentryReportUserError(const std::string& aggregate, const std::string& details)
+{
+    if (!static_sentryEnabled)
+        return;
+
+    sentry_value_t sentryMessage = sentry_value_new_object();
+    sentry_value_set_by_key(sentryMessage, "type", sentry_value_new_string("Error"));
+
+    sentry_value_t info = sentry_value_new_object();
+    sentry_value_set_by_key(info, "details", sentry_value_new_string(details.c_str()));
+
+    sentry_set_context("what", info);
+
+    sentry_value_t event = sentry_value_new_event();
+    sentry_value_set_by_key(event, "message", sentry_value_new_string(aggregate.c_str()));
+
+    sentry_capture_event(event);
+}
+
 } // of namespace
 
 #else
@@ -524,6 +553,11 @@ void addSentryTag(const char*, const char*)
 {
 }
 
+void updateSentryTag(const std::string&, const std::string&)
+{
+}
+
+
 void sentryReportNasalError(const std::string&, const string_list&)
 {
 }
@@ -540,6 +574,10 @@ void sentryThreadReportXMLErrors(bool)
 {
 }
 
+void sentryReportUserError(const std::string&, const std::string&)
+{
+}
+
 } // of namespace
 
 #endif
@@ -557,4 +595,14 @@ void addSentryTag(const std::string& tag, const std::string& value)
     addSentryTag(tag.c_str(), value.c_str());
 }
 
+SentryXMLErrorSupression::SentryXMLErrorSupression()
+{
+    sentryThreadReportXMLErrors(false);
+}
+
+SentryXMLErrorSupression::~SentryXMLErrorSupression()
+{
+    sentryThreadReportXMLErrors(true);
+}
+
 } // of namespace flightgear
