@@ -146,12 +146,16 @@ FlightPlan* FlightPlan::clone(const string& newIdent) const
       c->setCruiseFlightLevel(_cruiseFlightLevel);
   } else if (_cruiseAltitudeFt > 0) {
       c->setCruiseAltitudeFt(_cruiseAltitudeFt);
+  } else if (_cruiseAltitudeM > 0) {
+    c->setCruiseAltitudeM(_cruiseAltitudeM);
   }
 
   if (_cruiseAirspeedMach > 0) {
       c->setCruiseSpeedMach(_cruiseAirspeedMach);
   } else if (_cruiseAirspeedKnots > 0) {
       c->setCruiseSpeedKnots(_cruiseAirspeedKnots);
+  } else if (_cruiseAirspeedKph > 0) {
+    c->setCruiseSpeedKPH(_cruiseAirspeedKph);
   }
 
   c->_didLoadFP = true; // set the loaded flag to give delegates a chance
@@ -268,8 +272,10 @@ void FlightPlan::clearAll()
 
     _cruiseAirspeedMach = 0.0;
     _cruiseAirspeedKnots = 0;
+    _cruiseAirspeedKph = 0;
     _cruiseFlightLevel = 0;
     _cruiseAltitudeFt = 0;
+    _cruiseAltitudeM = 0;
 
     clearLegs();
     unlockDelegates();
@@ -655,12 +661,12 @@ void FlightPlan::setEstimatedDurationMinutes(int mins)
 
 void FlightPlan::computeDurationMinutes()
 {
-    if ((_cruiseAirspeedMach < 0.01) && (_cruiseAirspeedKnots < 10)) {
+    if ((_cruiseAirspeedMach < 0.01) && (_cruiseAirspeedKnots < 10) && (_cruiseAirspeedKph < 10)) {
         SG_LOG(SG_AUTOPILOT, SG_WARN, "can't compute duration, no cruise speed set");
         return;
     }
 
-    if ((_cruiseAltitudeFt < 100) && (_cruiseFlightLevel < 10)) {
+    if ((_cruiseAltitudeFt < 100) && (_cruiseAltitudeM < 100) && (_cruiseFlightLevel < 10)) {
         SG_LOG(SG_AUTOPILOT, SG_WARN, "can't compute duration, no cruise altitude set");
         return;
     }
@@ -808,12 +814,16 @@ void FlightPlan::saveToProperties(SGPropertyNode* d) const
         d->setIntValue("cruise/flight-level", _cruiseFlightLevel);
     } else if (_cruiseAltitudeFt > 0) {
         d->setIntValue("cruise/altitude-ft", _cruiseAltitudeFt);
+    } else if (_cruiseAltitudeM > 0) {
+        d->setIntValue("cruise/altitude-m", _cruiseAltitudeM);
     }
     
     if (_cruiseAirspeedMach > 0.0) {
         d->setDoubleValue("cruise/mach", _cruiseAirspeedMach);
     } else if (_cruiseAirspeedKnots > 0) {
         d->setIntValue("cruise/knots", _cruiseAirspeedKnots);
+    } else if (_cruiseAirspeedKph > 0) {
+        d->setIntValue("cruise/kph", _cruiseAirspeedKph);
     }
     
     // route nodes
@@ -1070,20 +1080,14 @@ bool FlightPlan::loadXmlFormat(const SGPath& path)
 {
   SGPropertyNode_ptr routeData(new SGPropertyNode);
 
-  // avoid error reports on user flight-plans
-  flightgear::sentryThreadReportXMLErrors(false);
   try {
     readProperties(path, routeData);
   } catch (sg_exception& e) {
      SG_LOG(SG_NAVAID, SG_ALERT, "Failed to load flight-plan '" << e.getOrigin()
              << "'. " << e.getMessage());
-    // XML parsing fails => not a property XML file
-    flightgear::sentryThreadReportXMLErrors(true);
     return false;
   }
 
-  flightgear::sentryThreadReportXMLErrors(true);
-
   if (routeData.valid())
   {
     try {
@@ -1187,12 +1191,16 @@ void FlightPlan::loadXMLRouteHeader(SGPropertyNode_ptr routeData)
           _cruiseFlightLevel = crs->getIntValue("flight-level");
       } else if (crs->hasChild("altitude-ft")) {
           _cruiseAltitudeFt = crs->getIntValue("altitude-ft");
+      } else if (crs->hasChild("altitude-m")) {
+          _cruiseAltitudeM = crs->getIntValue("altitude-m");
       }
 
       if (crs->hasChild("mach")) {
           _cruiseAirspeedMach = crs->getDoubleValue("mach");
       } else if (crs->hasChild("knots")) {
           _cruiseAirspeedKnots = crs->getIntValue("knots");
+      } else if (crs->hasChild("kph")) {
+          _cruiseAirspeedKph = crs->getIntValue("kph");
       }
   } // of cruise data loading
 }
@@ -1331,7 +1339,11 @@ bool FlightPlan::loadPlainTextFormat(const SGPath& path)
           return false;
       }
 
-      WayptRef w = waypointFromString(line);
+      SGGeod vicinity;
+      if (!_legs.empty()) {
+          vicinity = _legs.back()->waypoint()->position();
+      }
+      WayptRef w = waypointFromString(line, vicinity);
       if (!w) {
           SG_LOG(SG_NAVAID, SG_ALERT, "Failed to create waypoint from '" << line << "' in " << path);
           _legs.clear();
@@ -1354,182 +1366,23 @@ double FlightPlan::magvarDegAt(const SGGeod& pos) const
   double jd = globals->get_time_params()->getJD();
   return sgGetMagVar(pos, jd) * SG_RADIANS_TO_DEGREES;
 }
-  
 
-namespace
+WayptRef FlightPlan::waypointFromString(const string& tgt, const SGGeod& vicinity)
 {
-
-WayptRef intersectionFromString(FGPositionedRef p1,
-                                const SGGeod& basePosition,
-                                const double magvar,
-                                const string_list& pieces)
-{
-    assert(pieces.size() == 4);
-    // navid/radial/navid/radial notation
-    FGPositionedRef p2 = FGPositioned::findClosestWithIdent(pieces[2], basePosition);
-    if (!p2) {
-      SG_LOG( SG_NAVAID, SG_INFO, "Unable to find FGPositioned with ident:" << pieces[2]);
-      return nullptr;
-    }
-
-    double r1 = atof(pieces[1].c_str()),
-    r2 = atof(pieces[3].c_str());
-    r1 += magvar;
-    r2 += magvar;
-
-    SGGeod intersection;
-    bool ok = SGGeodesy::radialIntersection(p1->geod(), r1, p2->geod(), r2, intersection);
-    if (!ok) {
-      SG_LOG(SG_NAVAID, SG_INFO, "no valid intersection for:" << pieces[0]
-              << "/" << pieces[2]);
-      return NULL;
-    }
-
-    std::string name = p1->ident() + "-" + p2->ident();
-    return new BasicWaypt(intersection, name, nullptr);
-}
-
-WayptRef wayptFromLonLatString(const std::string& target)
-{
-    // permit lat,lon/radial/offset format
-    string_list pieces(simgear::strutils::split(target, "/"));
-    if ((pieces.size() != 1) && (pieces.size() != 3)) {
-        return {};
-    }
-    
-    SGGeod g;
-    const bool defaultToLonLat = true; // parseStringAsGeod would otherwise default to lat,lon
-    if (!simgear::strutils::parseStringAsGeod(pieces[0], &g, defaultToLonLat)) {
-        return {};
-    }
-    
-    if (pieces.size() == 3) {
-        // process offset
-        const double bearing = std::stod(pieces[1]);
-        const double distanceNm = std::stod(pieces[2]);
-        g = SGGeodesy::direct(g, bearing, distanceNm * SG_NM_TO_METER);
-    }
-    
-    // build a short name
-    const int lonDeg = static_cast<int>(g.getLongitudeDeg());
-    const int latDeg = static_cast<int>(g.getLatitudeDeg());
-
-    char buf[32];
-    char ew = (lonDeg < 0) ? 'W' : 'E';
-    char ns = (latDeg < 0) ? 'S' : 'N';
-    snprintf(buf, 32, "%c%03d%c%03d", ew, std::abs(lonDeg), ns, std::abs(latDeg));
-
-    return new BasicWaypt(g, buf, nullptr);
-}
-
-WayptRef viaFromString(const SGGeod& basePosition, const std::string& target)
-{
-    assert(target.find("VIA ") == 0);
-    string_list pieces(simgear::strutils::split(target, "/"));
-    if ((pieces.size() != 4) || (pieces[2] != "TO")) {
-        SG_LOG( SG_NAVAID, SG_WARN, "Malformed VIA specification string:" << target);
-        return nullptr;
-    }
-    
-    // TO navaid is pieces[3]
-    FGPositionedRef nav = FGPositioned::findClosestWithIdent(pieces[3], basePosition, nullptr);
-    if (!nav) {
-        SG_LOG( SG_NAVAID, SG_WARN, "TO navaid:" << pieces[3] << " unknown");
-        return nullptr;
-    }
-    
-    // airway ident is pieces[1]
-    AirwayRef airway = Airway::findByIdentAndNavaid(pieces[1], nav);
-    if (!airway) {
-        SG_LOG( SG_NAVAID, SG_WARN, "Unknown airway:" << pieces[1]);
-        return nullptr;
-    }
-
-    return new Via(nullptr, airway, nav);
-}
-
-} // of anonymous namespace
-
-WayptRef FlightPlan::waypointFromString(const string& tgt )
-{
-  string target = simgear::strutils::uppercase(tgt);
-  // extract altitude
-  double altFt = 0.0;
-  RouteRestriction altSetting = RESTRICT_NONE;
-  
-  size_t pos = target.find( '@' );
-  if ( pos != string::npos ) {
-    altFt = atof( target.c_str() + pos + 1 );
-    target = target.substr( 0, pos );
-    if ( !strcmp(fgGetString("/sim/startup/units"), "meter") )
-      altFt *= SG_METER_TO_FEET;
-    altSetting = RESTRICT_AT;
-  }
-  
-  // check for lon,lat
-    WayptRef wpt = wayptFromLonLatString(target);
-
-  SGGeod basePosition;
-  if (_legs.empty()) {
-    // route is empty, use departure position / aircraft position
-    basePosition = _departure ? _departure->geod() : globals->get_aircraft_position();
-  } else {
-    basePosition = _legs.back()->waypoint()->position();
-  }
-  
-  const double magvar = magvarDegAt(basePosition);
-
-  if (wpt) {
-      // already handled in the lat/lon test above
-  } else if (target.find("VIA ") == 0) {
-      wpt = viaFromString(basePosition, target);
-  } else {
-      string_list pieces(simgear::strutils::split(target, "/"));
-      FGPositionedRef p = FGPositioned::findClosestWithIdent(pieces.front(), basePosition);
-      if (!p) {
-        SG_LOG( SG_NAVAID, SG_INFO, "Unable to find FGPositioned with ident:" << pieces.front());
-        return nullptr;
-      }
-
-      if (pieces.size() == 1) {
-        wpt = new NavaidWaypoint(p, this);
-      } else if (pieces.size() == 3) {
-        // navaid/radial/distance-nm notation
-        double radial = atof(pieces[1].c_str()),
-        distanceNm = atof(pieces[2].c_str());
-        radial += magvar;
-        wpt = new OffsetNavaidWaypoint(p, this, radial, distanceNm);
-      } else if (pieces.size() == 2) {
-        FGAirport* apt = dynamic_cast<FGAirport*>(p.ptr());
-        if (!apt) {
-          SG_LOG(SG_NAVAID, SG_INFO, "Waypoint is not an airport:" << pieces.front());
-          return nullptr;
+    SGGeod basePosition = vicinity;
+    if (!vicinity.isValid()) {
+        // compute basePosition using some heuristics
+        if (_legs.empty()) {
+            // route is empty, use departure position / aircraft position
+            basePosition = _departure ? _departure->geod() : globals->get_aircraft_position();
+        } else {
+            basePosition = _legs.back()->waypoint()->position();
         }
+    }
 
-        if (!apt->hasRunwayWithIdent(pieces[1])) {
-          SG_LOG(SG_NAVAID, SG_INFO, "No runway: " << pieces[1] << " at " << pieces[0]);
-          return nullptr;
-        }
-
-        FGRunway* runway = apt->getRunwayByIdent(pieces[1]);
-        wpt = new NavaidWaypoint(runway, this);
-      } else if (pieces.size() == 4) {
-        wpt = intersectionFromString(p, basePosition, magvar, pieces);
-      }
-  }
-  
-  if (!wpt) {
-    SG_LOG(SG_NAVAID, SG_INFO, "Unable to parse waypoint:" << target);
-    return nullptr;
-  }
-  
-  if (altSetting != RESTRICT_NONE) {
-    wpt->setAltitude(altFt, altSetting);
-  }
-  return wpt;
+    return Waypt::createFromString(this, tgt, basePosition);
 }
 
-
 void FlightPlan::activate()
 {
   FGRouteMgr* routeManager = globals->get_subsystem<FGRouteMgr>();
@@ -2215,9 +2068,10 @@ ICAOFlightType FlightPlan::flightType() const
 void FlightPlan::setCruiseSpeedKnots(int kts)
 {
     lockDelegates();
+    _cruiseDataChanged = true;
     _cruiseAirspeedKnots = kts;
     _cruiseAirspeedMach = 0.0;
-    _cruiseDataChanged = true;
+    _cruiseAirspeedKph = 0;
     unlockDelegates();
 }
 
@@ -2230,8 +2084,9 @@ void FlightPlan::setCruiseSpeedMach(double mach)
 {
     lockDelegates();
     _cruiseDataChanged = true;
-    _cruiseAirspeedMach = mach;
     _cruiseAirspeedKnots = 0;
+    _cruiseAirspeedMach = mach;
+    _cruiseAirspeedKph = 0;
     unlockDelegates();
 }
 
@@ -2240,12 +2095,28 @@ double FlightPlan::cruiseSpeedMach() const
     return _cruiseAirspeedMach;
 }
 
+void FlightPlan::setCruiseSpeedKPH(int kph)
+{
+    lockDelegates();
+    _cruiseDataChanged = true;
+    _cruiseAirspeedKnots = 0;
+    _cruiseAirspeedMach = 0.0;
+    _cruiseAirspeedKph = kph;
+    unlockDelegates();
+}
+
+int FlightPlan::cruiseSpeedKPH() const
+{
+    return _cruiseAirspeedKph;
+}
+
 void FlightPlan::setCruiseFlightLevel(int flightLevel)
 {
     lockDelegates();
     _cruiseDataChanged = true;
-    _cruiseFlightLevel = flightLevel;
     _cruiseAltitudeFt = 0;
+    _cruiseAltitudeM = 0;
+    _cruiseFlightLevel = flightLevel;
     unlockDelegates();
 }
 
@@ -2259,6 +2130,7 @@ void FlightPlan::setCruiseAltitudeFt(int altFt)
     lockDelegates();
     _cruiseDataChanged = true;
     _cruiseAltitudeFt = altFt;
+    _cruiseAltitudeM = 0;
     _cruiseFlightLevel = 0;
     unlockDelegates();
 }
@@ -2268,6 +2140,21 @@ int FlightPlan::cruiseAltitudeFt() const
     return _cruiseAltitudeFt;
 }
 
+void FlightPlan::setCruiseAltitudeM(int altM)
+{
+    lockDelegates();
+    _cruiseDataChanged = true;
+    _cruiseAltitudeFt = 0;
+    _cruiseAltitudeM = altM;
+    _cruiseFlightLevel = 0;
+    unlockDelegates();
+}
+
+int FlightPlan::cruiseAltitudeM() const
+{
+    return _cruiseAltitudeM;
+}
+
 void FlightPlan::forEachLeg(const LegVisitor& lv)
 {
     std::for_each(_legs.begin(), _legs.end(), lv);
