@@ -29,19 +29,20 @@
 #include <osg/BlendFunc>
 
 #include <osg/NodeVisitor>
-#include <osgUtil/CullVisitor>
-#include <osgGA/GUIEventHandler>
 #include <osgGA/GUIEventAdapter>
-
-#include <plib/pu.h>
+#include <osgGA/GUIEventHandler>
+#include <osgUtil/CullVisitor>
+#include <osgViewer/Viewer>
 
 #include <simgear/scene/util/SGNodeMasks.hxx>
 
+#include <GUI/FlightGear_pu.h>
 #include <Main/fg_props.hxx>
 #include <Main/globals.hxx>
 #include <Main/locale.hxx>
 #include <Viewer/CameraGroup.hxx>
 #include <Viewer/FGEventHandler.hxx>
+#include <Viewer/renderer.hxx>
 
 #include <Input/input.hxx>
 #include <Input/FGMouseInput.hxx>
@@ -167,7 +168,9 @@ public:
             case(osgGA::GUIEventAdapter::PUSH):
             case(osgGA::GUIEventAdapter::RELEASE):
             {
-                FGMouseInput* mouseSubsystem = globals->get_subsystem<FGInput>()->get_subsystem<FGMouseInput>();
+                // during splash/reset, either of these can return nullptr
+                const auto input = globals->get_subsystem<FGInput>();
+                const auto mouseSubsystem = input ? input->get_subsystem<FGMouseInput>() : nullptr;
                 if (mouseSubsystem && !mouseSubsystem->isActiveModePassThrough()) {
                     return false;
                 }
@@ -301,7 +304,20 @@ PUICamera::PUICamera() :
 {
 }
 
-void PUICamera::init(osg::Group* parent)
+PUICamera::~PUICamera()
+{
+    SG_LOG(SG_GL, SG_INFO, "Deleting PUI camera");
+
+    // depending on if we're doing shutdown or reset, various things can be
+    // null here.
+    auto renderer = globals->get_renderer();
+    auto viewer = renderer ? renderer->getViewer() : nullptr;
+    if (viewer) {
+        viewer->removeEventHandler(_eventHandler);
+    }
+}
+
+void PUICamera::init(osg::Group* parent, osgViewer::Viewer* viewer)
 {
     setName("PUI FBO camera");
     
@@ -363,11 +379,6 @@ void PUICamera::init(osg::Group* parent)
     fsQuadGeode->addDrawable(_fullScreenQuad);
     fsQuadGeode->setName("PUI fullscreen Geode");
     fsQuadGeode->setNodeMask(SG_NODEMASK_GUI_BIT);
-    
-    // set the event callback on the Geode; if we set it on the Drawable,
-    // osgGA calls it twice (as a NodeCallback and also a Drawable::EventCallback)
-    fsQuadGeode->setEventCallback(new PUIEventHandler(this));
-
     parent->addChild(this);
     parent->addChild(fsQuadGeode);
     
@@ -376,6 +387,12 @@ void PUICamera::init(osg::Group* parent)
         osg::Viewport* vport = camera->getViewport();
         resizeUi(vport->width(), vport->height());
     }
+
+    // push_front so we keep the order of event handlers the opposite of
+    // the rendering order (i.e top-most UI layer has the front-most event
+    // handler)
+    _eventHandler = new PUIEventHandler(this);
+    viewer->getEventHandlers().push_front(_eventHandler);
 }
 
 // remove once we require OSG 3.4
@@ -400,12 +417,13 @@ void PUICamera::resizeUi(int width, int height)
     resizeAttachments(scaledWidth, scaledHeight);
 #endif
 
+    const float puiZ = 1.0;
     // resize the full-screen quad
     osg::Vec3Array* fsQuadVertices = static_cast<osg::Vec3Array*>(_fullScreenQuad->getVertexArray());
-    (*fsQuadVertices)[0] = osg::Vec3(0.0, height, 0.0);
-    (*fsQuadVertices)[1] = osg::Vec3(0.0, 0.0, 0.0);
-    (*fsQuadVertices)[2] = osg::Vec3(width, 0.0, 0.0);
-    (*fsQuadVertices)[3] = osg::Vec3(width, height, 0.0);
+    (*fsQuadVertices)[0] = osg::Vec3(0.0, height, puiZ);
+    (*fsQuadVertices)[1] = osg::Vec3(0.0, 0.0, puiZ);
+    (*fsQuadVertices)[2] = osg::Vec3(width, 0.0, puiZ);
+    (*fsQuadVertices)[3] = osg::Vec3(width, height, puiZ);
 
     fsQuadVertices->dirty();
 }
